/* Generate multiple graphs with false-positive and false-negative
   error curves using different values of PctTrue, ntotal, and 
   stddev or using a different power function.

   After running this program, you can print the Results window
   to a PDF file, which causes each graph to appear on a single 
   page. It is then easy to page through the document to watch how 
   the position of the arrow showing the optimal threshold p-value 
   changes as the parameters change.  

   Don Macnaughton July 2022  */

title;
options nodate;


/* If you run this program in SAS OnDemand for Academics, the 
   following ods html; statement will cause an error saying:
      Insufficient authorization ...
      No body file. HTML output will not be created.
   You can ignore this error becuase the proper HTML output will be 
   created when the program actually starts generating output. */

ods html;
ods text="Output from Program BowlGraphsExamples.sas";
ods text=" ";
ods text="This ouput is from a program in the Supplementary Information for the paper ""The Optimal Threshold p-Value for a Scientific Journal"" by Donald B. Macnaughton.";
ods text=" ";
ods text="This output is easiest to understand if it is viewed in a PDF reader that can display a full page and that can turn pages so that each page exactly overlays the preceding page when the page is turned. Under those conditions you can turn the pages quickly and see how the vertical arrow on the pages moves around. Otherwise, you can drag the vertical slider to quickly scroll through the graphs.";
ods text=" ";
ods text="The bowl depicted by the black line on each graph shows the sum of the costs of the false-positive and false-negative errors as a function of the threshold p-value. The lowest point on the bowl (identified by the downward-pointing arrow on each graph) indicates the optimal threshold p-value for the studied journal in this case. We see how the optimal threshold p-value changes as the values of the parameters of the situation change. The values of the changing parameters of the situation depicted on each graph are given in the inset in the lower-left corner of the graph.";
ods text=" ";
ods text="The following 15 graphs reflect three series, with five graphs in each series. In the first series, the percentage of research hypotheses in the field that are true is assumed to be 40%, as indicated by PctTrue in the inset on each graph. In the second series, the percentage true is assumed to be 50%, and in the third, the percentage true is assumed to be 60%. The five graphs in each series show the correct graph when the standard deviation has each of five different values, ranging from 15 up to 500, as indicated by the StdDev value in the inset. The value of meandiff is 20. So, if the standard deviation is 500, then the effect is very small relative to the noise, making it effectively non-existent--i.e., there is effectively no relationship between the two variables.";
ods text=" ";
ods text="The main takeaway from these graphs is that the optimal threshold p-value moves around depending on the values of the parameters.";
ods text=" ";
ods text="It is easy rerun this program with different groups of values of the parameters to see what happens. To do that, find the program in the Supplementary Information for the paper, change the specifications of the three DO statements at the beginning of the program, and then run the program. Instructions to run SAS programs using free online SAS software are in the Supplementary Information for the paper.";
ods text=" ";
ods text=" ";
ods text=" ";
ods text=" ";
ods text=" ";
ods text=" ";

data one;

retain PosPctPub 100 meandiff 20;

/* It is possible to change the nature of relationships that are 
   depicted on the graphs with either of the following aproaches:
   1. Change the specifications of the three DO statements that
      follow immediately below. The enables varying the parameters
      of the two-sample t-test to see on the graphs how the changes
      affect the specification of the optimal threshold p-value on
      the graphs.
   2. Change the three lines that define the power equations for the
      research study to appropriate lines for some other reserch 
      study of interest and add approriate text to the RETAIN 
      statement specifying the required parameter values to support 
      the new power equations. This illustrates how the program can
      in theory be used to draw the appropriate graph for any 
      research study that we might wish to consider.
*/

GroupNo = 0;
do PctTrue = 40, 50, 60;  
   do ntotal = 60; 
      do stddev= 15, 27, 35, 250, 500;

GroupNo = GroupNo + 1;

do ThreshP = 0.0001 to 0.1 by 0.0001, 0.101 to 0.99 by 0.001;

   nTrueRels = 1000 * (PctTrue / 100);
   nFalseRels = 1000 - nTrueRels;
   nFalsePos = nFalseRels * ThreshP;
   nFalsePosPub = nFalsePos * PosPctPub/100;

/* Here are the three statements to compute the power of the two-sample 
   t-test, as specified at 
   https://documentation.sas.com/doc/en/pgmsascdc/9.4_3.2/statug/statug_intropss_sect017.htm  */
   Ncp = ntotal * 0.5 * 0.5 * meandiff**2 / stddev**2;
   Critval = finv(1-ThreshP, 1, ntotal-2, 0);
   TestPower = sdf('f', critval, 1, ntotal-2, ncp); 

   nTruePos = nTrueRels * TestPower;
   nFalseNeg = nTrueRels - nTruePos;

/* Compute the ordinates for the three lines on the graph. */
   PropFalsePosPub = nFalsePosPub / 1000;
   PropFalseNeg = nFalseNeg / 1000;

   Cost = PropFalsePosPub + PropFalseNeg;

/* The following variables aren't plotted, but they could be plotted. */
   PropTruePos = nTruePos / 1000;
   PropPubFalsePos = nFalsePosPub / ((nTruePos * PosPctPub/100) + nFalsePosPub);
   output;
   end;

end; end; end;

label ThreshP = "Threshold p-value"
      TestPower = "Power of the test"
      PropFalsePosPub = "Proportion of all results that are published false-positive errors"
      PropFalseNeg  = "Proportion of all results that are false-negative errors"
      PropTruePos = "Proportion of all results that are true positive"
      Cost = "Relative total cost of the errors assuming equal unit costs of FP and FN errors"
      PropPubFalsePos = "Proportion of published results that are false-positive";
run;

/* Get the number of graphs to be plotted from finding the highest 
   group number in the data file and save it in the NGroups macro 
   variable for later use. */
proc sql noprint;
   select max(GroupNo) into :NGroups TRIMMED from one;
   quit;

ods graphics / noborder;
%sganno;

/* Define a SAS macro, that will repeatedly execute the code to 
   generate the plots using the appropriate data each time (as 
   selected by GroupNo). */

%macro DoPlots();

%do Group = 1 % to &NGroups;

%if &Group=1 %then %do;
   %put _user_;
%end;

/* Initialize macro variables that may fail at assignment with 
   dashes. (Macro variables have character values.) Thus any 
   assignement failures will be identified in the inset on the 
   graphs as dashes. */
%let ThreshP6 = -;
%let PropPubFalsePos6 = -;
%let Count6 = -;

/* Extract some required values from the data table into macro 
   variables. These values will be used in annotating the graphs
   in various ways. */
proc sql noprint;
   select min(PctTrue) format=2., min(ntotal) format=3., 
      min(stddev) format=5., min(PosPctPub) format=3.
      into :PctTrue TRIMMED, :ntotal TRIMMED, :stddev TRIMMED, :PosPctPub TRIMMED from one
      where GroupNo = &Group;

/* Get information needed to draw the arrow. */
   select Cost, ThreshP into :minCost TRIMMED, :OptThresh TRIMMED
      from one 
      where GroupNo = &Group and 
         Cost = select min(Cost) from one where GroupNo = &Group;

quit;

/* proc sgplot data=one (where=(GroupNo = &Group));
   series x=ThreshP y = TestPower;
   run; */

/* Generate data to draw the arrow on the plot. */
data anno1;
   %sgarrow(drawspace="datavalue", x1=&OptThresh, y1=&minCost, 
      x2=&OptThresh, y2=0, linecolor="black", linethickness=1,
      layer="back");
   run;

/* Plot the error rates and the sum of the error rates on the same graph. */
proc sgplot data=one (where=(GroupNo=&Group)) sganno=anno1 nowall;
   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red);
   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
/* series x=ThreshP y=PropTruePos / lineattrs=(color=purple); */
   series x=ThreshP y=Cost / lineattrs=(color=black);
   xaxis type=log logbase=10 logstyle=linear label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value"; */
   yaxis label = "Proportion of all results";
   keylegend / location=inside position=top opaque;
   inset "PctTrue = &PctTrue"
         "StdDev = &stddev"
      /* "PosPctPub = &PosPctPub"
         "ntotal = &ntotal" */

      / position = bottomleft border backcolor=white valuealign=left;
   run;

%end;

%mend;

/* Now that the doplots macro has been defined, execute it to 
   generate the graphs. */
%doplots;
