/* This SAS program generates graphs with false-positive and false-
   negative error curves as a function of a journal's threshold p-
   value for a specific two-sample t-test case. This is the main 
   version of the program.

   Don Macnaughton June 2022

   This discussion assumes that you have read the paper "The Optimal
   Threshold p-Value for a Scientific Journal" including appendix B
   "The Existence of the Optimal Threshold p-Value for a Journal".
   This program generates the data behind figure B6 in the paper and
   then shows graphs of various views of the data to help to 
   illustrate how the ideas work.

   Note that this version of the computations assumes that false-
   positive and false-negative errors are equally costly, so we can 
   simply add together the two error rates to get the sum of the 
   costs (on a relative cost scale, which is all we need). The 
   program can be easily adjusted if the error costs require a more 
   complicated treatment than that.

   The following DATA step code is an enhanced version of the program
   given in Appendix M of the paper. The enhancements are that this 
   version of the program
      1. generates a wider range of threshold p-values to allow us to
         see a wider picture of what goes on
      2. generates a few extra variables to support points that are 
         explained in the program output. */
data GraphData; 

/* In case the following values are changed, the "standard" values 
   for the following statement are 30, 90, 60, 20, and 27. */
retain PctTrue 30 PosPctPub 90 ntotal 60 meandiff 20 stddev 27;

/* Note the broad range of threshold values being generated by the
   program, from 0.00001 to 1.0. Of course Figures M.1 - M.6 in 
   appendix M are only showing a small part of this range, from 
   0.01 to 0.3 */
do ThreshP = 0.00001 to 0.0001 by 0.00001, 0.0002 to 1.0 by 0.0001;
   nTrueRels = 1000 * (PctTrue/100);
   nFalseRels = 1000 - nTrueRels; 
   nFalsePos = ThreshP * nFalseRels; 
   nFalsePosPub = nFalsePos * (PosPctPub/100); 

/* The following three lines can be replaced by other appropriate 
   lines of code to compute TestPower as a function of the threshold
   p-value for a different research design. Thus the program can be 
   easily modified to model any research situation that computes a 
   standard p-value. */
   Ncp = ntotal * 0.5 * 0.5 * meandiff**2 / stddev**2;
   Critval = finv(1-ThreshP, 1, ntotal-2, 0);
   TestPower = sdf('f', Critval, 1, ntotal-2, Ncp); 

   nTruePos = TestPower * nTrueRels;
   nFalseNeg = nTrueRels - nTruePos;

   PropFalsePosPub = nFalsePosPub / 1000;
   PropFalseNeg = nFalseNeg / 1000;
   PropTruePos = nTruePos / 1000; 
   PropTrueNeg = (nFalseRels - nFalsePos) / 1000; 

/* Compute the relative total cost of the errors, assuming that 
   false-positive and false-negative errors have equal costs of 1.0
   per unit of probability. */
   Cost = PropFalsePosPub + PropFalseNeg;

/* Compute some other variables for checking. */
   Benefit = PropTruePos + PropTrueNeg;
   SumCostBen = round(Cost + Benefit,.0000001);
   
/* Write all the variable values in this observation to a new
   row in the "GraphData" data set. */
   output;
   end; /* I.e., the end of the DO loop */

label ThreshP = "Threshold p-value"
      TestPower = "Power of test"
      PropFalsePosPub = "Proportion of all results that are false-positive errors and are published"
      PropFalseNeg  = "Proportion of all results that are (unpublished) false-negative errors"
      PropTruePos = "Proportion of all results that are true positive"
      PropTrueNeg = "Proportion of all results that are true negative"
      Cost = "Relative total cost of errors assuming equal unit costs of FP and FN errors"
      Benefit = "Sum of proportions of TN and TP results = benefit"
      SumCostBen = "Sum of costs and benefits"
      ;

/* The following line indicates that the specification of the data 
   step is complete and SAS can now "run" the data step. */
run;

title;
options nodate;

/* Use the SQL query language to extract some specific values from 
   the new GraphData data set and put the values "into" SAS "macro 
   variables" for later use in adding annotations to the graphs and 
   in text output from the program. Note that the values of macro 
   variables in SAS are CHARACTER STRINGS. */
proc sql noprint;
/* First, get the minimum cost and the threshold p-value for that 
   cost, which is the optimal threshold p-value. Note the elegant
   simplicity of the last line in the following SELECT statement.
   Get the value of the optimal threshold p-value in two formats:
   with four digits after the decimal point (for higher accuracy) 
   and with two digits after the decimal point (for compact 
   communication). */
   select   Cost,             ThreshP format=8.4, 
         ThreshP format=6.2 
      into :minCost TRIMMED, :ThreshPminCost TRIMMED,    
         :ThreshPminCost2 TRIMMED
      from GraphData 
      where Cost = select min(Cost) from GraphData;

/* Get the power of the test when the threshold p-value is 0.05
   with 15 and 3 digits after the decimal point. */
   select   TestPower format=20.15, TestPower format=20.3 
      into :TestPower05 TRIMMED,   :TestPower053 TRIMMED
      from GraphData
      where Round(ThreshP, 0.00001) = 0.05;

/* Get the minimum power of the test to define the y-
   coordinate of the lower end of the vertical straight line on 
   the next graph below. */
   select   min(TestPower),         min(ThreshP) 
      into :minTestPower TRIMMED,  :minThreshP  TRIMMED
      from GraphData
      where ThreshP >= 0.01 and ThreshP <= 0.30;

/* Get the maximum benefit to define the y-coordinate of the 
   maximum. */
   select max(Benefit) into :maxBenefit TRIMMED from GraphData;

/* Get the y-coordinate of the minimum point on the purple line for
   the tip of the arrowhead on the TP & TN graph. */
   select case 
      when (select min(PropTruePos) from GraphData) < 0.1 then 0
      else min(PropTruePos)
   end as temp into :minPropTruePos TRIMMED from GraphData;

/* Get the values of PosPctPub and PctTrue. These values are constant
   throughout a run of the program. */
   select min(PosPctPub), min(PctTrue) 
      into :PosPctPub TRIMMED, :PctTrue TRIMMED from GraphData;

quit;

/* Compute with floating-point arithmetic some macro variable values
   for use in the output. */
%let PropTrue = %sysevalf(&PctTrue/100);
%let PropFalse = %sysevalf(1 - &PctTrue/100);
%let PctFalse = %sysevalf(100 * &PropFalse);

/* The following statement prints the values of all the macro 
   variables in the program log for checking. Note that with the 
   standard settings in the RETAIN statement at the beginning of
   the program, the value of TestPower05 when the threshold p-value
   is 0.05 is given in the log, as printed by the %put _user_ 
   statement as 0.805412.... As a consistency check, this value is
   also given from the code that follows below using PROC POWER. */
%put _user_;

/* Use the Output Delivery System to turn off the printed ouput from
   the following procedure because the standard output doesn't show 
   enough significant digits for the reported power. */
ods select none; 

/* Compute the power of the specified two-sample t-test. 
   Note that the values of meandiff, stddev, and npergroup are 
   hardcoded below as 20, 27, and 30 for ease of understanding. If 
   you change the values of these variables in the RETAIN statement 
   above, they should also be changed here to maintain consistency.
   ntotal in the RETAIN statement = 2 x npergroup.  */
proc power;
   twosamplemeans test=diff sides=2 meandiff=20 stddev=27  
      npergroup=30 alpha=0.05 nfractional power=.;
/* The following following Output Delivery System statement tells SAS 
   to put the output from the run of PROC POWER in a new data set 
   called PowerData. */
   ods output Output=PowerData; 
run;

ods text="Computer Output from Program BowlGraphMain.sas";
ods text=" ";
ods text="This output illustrates key ideas behind the paper ""The Optimal Threshold p-value for a Scientific Journal"". The focus is on a two-sample t-test and on the false-positive and false-negative errors that the test makes.";
ods text=" ";
ods text= "We examine the performance of the two-sample t-test in a specific case in which we assume that the population difference between the means is 20 units and the population standard deviation of the difference is 27 units. We assume that there are 30 entities in each of the two groups for a total of 60 entities in the research. These assumptions enable us to compute the ""power"" of the test. Here, computed two different ways, is the power of the test when the threshold p-value (which SAS refers to as ""Alpha"") is 0.05:";
ods text=" ";
ods text="First is the output from the SAS Power procedure, which informs us that it is using the conditions specified above and then tells us the power of the test under the specified conditions.";
ods text=" ";
/* Print the PowerData data set with 15 significant digits for the
   Power value to allow closer comparison with the number computed 
   directly by this program. */
ods select all; 
proc print data=PowerData noobs; 
   format Power 20.15; 
run;
ods text="Next, to check for consistency, we can compare the power above with the power computed directly by the three lines of code in this program when the threshold p-value is 0.05, which (as extracted from the data file of this run of the program) is &TestPower05..";
ods text=" ";
ods text="Note that the values 20, 27, and 2 x 30 = 60 were intentionally chosen to make the power roughly 0.8 when the threshold p-value is 0.05. This choice was made because it makes the situation under study resemble the ideal typical situation in which the power of the test is 0.8 for detecting the expected form of the studied effect. Of course, even higher power would be better, but that would be costly, so we must compromise to control research costs.";

/* Enable the "annotate macros" which facilitate adding custom lines,
   shapes, and annotations to graphs. */
%sganno;

/* The next code specifies the horizontal and vertical straight 
   lines that appear on the next graph using the values that were put
   in the macro variable in the PROC SQL procedure above. */
data anno1;
   %sgline(drawspace="datavalue", x1=0.05, y1=&TestPower05, 
      x2=&minThreshP, y2=&TestPower05, linecolor="black", linethickness=1);
   %sgline(drawspace="datavalue", x1=0.05, y1=&TestPower05, 
      x2=0.05, y2=&minTestPower, linecolor="black", linethickness=1);
   run;
   
/* Plot the power of the test against the threshold p-value to
   monitor the operation of the program. */
ods graphics / noborder;
proc sgplot data=GraphData sganno=anno1;
   where ThreshP >= 0.01 and ThreshP <= 0.30;
   series x=ThreshP y = TestPower; 
/* xaxis values=(0 to 0.3 by 0.05); */
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
   run;
ods text="The above graph indicates the power of the two-sample t-test computed at each threshold p-value between 0.01 and 0.3, with the value at 0.05 highlighted, showing that the power of the test at this threshold p-value is roughly &TestPower053., as noted above.";
ods text=" ";
ods text="The graph tells us the power of the test under the specified conditions (i.e., mean difference of 20, standard deviation of 27, number in each group of 30). However, ANY other standard statistical test for evidence of an effect will have a similar smoothly monotonically increasing line on the graph, though the location and shape of the line on the graph will vary from test to test. The relationship between the threshold p-value and the power is of interest because this relationship determines the locations of the green lines on the graphs below.";

/* The next code specifies the arrow on the following graphs using 
   the values obtained in the PROC SQL procedure above. */
data anno2;
   %sgarrow(drawspace="datavalue", x1=&ThreshPminCost, y1=&minCost, 
      x2=&ThreshPminCost, y2=0, linecolor="black", linethickness=1);
   run;

/* Plot the error rates and the sum of the error rates on the same 
   graph. */
/* Purple and green balance, as do red and orange. */
proc sgplot data=GraphData  sganno=anno2;
/* where ThreshP >= 0.01 and ThreshP <= 0.30;  */
   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); 
   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=Cost / lineattrs=(color=black); 

/* series x=ThreshP y=PropTruePos / lineattrs=(color=purple); 
   series x=ThreshP y=PropTrueNeg / lineattrs=(color=orange); 
   series x=ThreshP y=Benefit / lineattrs=(color=black pattern=mediumdash); */
   keylegend / location=inside position=top opaque; 
/*   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; */
   xaxis type=linear label = "Threshold p-value (LINEAR scale)"; 
   yaxis label = "Proportion of all results";
   run;
ods text="Here is figure M.6 from the paper, but with a LINEAR horizontal axis as opposed to a logarithmic horizontal axis, and using (almost) the entire possible range of potential threshold p-values between 0.0 and 1.0. Note how the false-postitive line (red line) is a straight-line relationship, as can also be seen in the simple algebra in the program that generates the data behind the line.";
ods text=" ";
ods text="On the above and following graphs, the vertical arrow that runs down from the minimum point on the curving black line shows the theoretical optimal threshold p-value of roughly &ThreshPminCost2 for the journal for this specific research situation. This will become clearer when we zoom in on the arrow, which we do in a moment.";

proc sgplot data=GraphData  sganno=anno2 ;
   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); 
   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=Cost / lineattrs=(color=black); 

/* series x=ThreshP y=PropTruePos / lineattrs=(color=purple); 
   series x=ThreshP y=PropTrueNeg / lineattrs=(color=orange); 
   series x=ThreshP y=Benefit / lineattrs=(color=black pattern=mediumdash); */
   keylegend / location=inside position=top opaque; 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value (LINEAR scale)"; */
   yaxis label = "Proportion of all results";
   run;

ods text="The above graph shows exactly the same data as the graph that precedes it, but using a logarithmic scale for the horizontal axis instead of a linear scale. This scale sensibly stretches things out at the lower end. Note how changing the axis scale has changed the false-positive (red) line from a straight line to a curved line. Note how changing the axis scale DOESN'T change the location on the horizontal axis of the minimum point of the bowl of roughly &ThreshPminCost2..";
ods text=" ";
ods text="Note how the false-negative (green) line is a sigmoid function, but the false-positive (red) line is not.";
ods text=" ";
ods text="Note how the black line theoretically peaks at the left end at &PropTrue, as defined by the value of PctTrue in the program. The black line peaks at the right end at 1 - &PropTrue = &PropFalse if PosPctPub is 100. It peaks at a little less than &PropFalse if PosPctPub is less than 100. The current value of PosPctPub in this program is &PosPctPub..";
ods text=" ";
ods text="All the following graphs use a logarithmic scale on the horizontal axis, but we could draw the same conclusions if the scale were a linear scale, though data at the low end of the horizontal axis would be somewhat squished on the graphs in that case.";
proc sgplot data=GraphData  sganno=anno2 ;
   where ThreshP >= 0.01 and ThreshP <= 0.30;
   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); 
   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=Cost / lineattrs=(color=black); 

/* series x=ThreshP y=PropTruePos / lineattrs=(color=purple); 
   series x=ThreshP y=PropTrueNeg / lineattrs=(color=orange); 
   series x=ThreshP y=Benefit / lineattrs=(color=black pattern=mediumdash); */
   keylegend / location=inside position=top opaque; 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value (LINEAR scale)"; */
   yaxis label = "Proportion of all results";
   run;

ods text="The above graph is figure M.6 in the paper. This graph is based on the same data as on the earlier graphs with the red and green lines. However, this graph is zoomed in on the small area of the horizontal axis of main interest. We can see more clearly here how the lowest point on the bowl is at roughly &ThreshPminCost2 on the horizontal axis. (The actual optimal threshold p-value extracted from the data file is &ThreshPminCost..)";

data anno3;
   %sgarrow(drawspace="datavalue", x1=&ThreshPminCost, y1=&maxBenefit, 
      x2=&ThreshPminCost, y2=0, linecolor="black", linethickness=1,
      layer="back");
   run;

proc sgplot data=GraphData nowall sganno=anno3 ;
   where ThreshP >= 0.01 and ThreshP <= 0.30;
/*   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); 
     series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=Cost / lineattrs=(color=black); */

   refline .3 .7;
   series x=ThreshP y=Benefit / lineattrs=(color=black); 
   series x=ThreshP y=PropTrueNeg / lineattrs=(color=purple); 
   series x=ThreshP y=PropTruePos / lineattrs=(color=orange); 
   inset "Horizontal lines are at y=0.3 and y=0.7." / position=bottomleft 
      border opaque;
   keylegend / location=inside position=left opaque; 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value"; */
   yaxis min=0 label = "Proportion of all results";
   run;
ods text="The above graph shows that we get the same optimum threshold p-value of roughly &ThreshPminCost2 on the horizontal axis by finding the point that maximizes the sum of the ""benefits"" from the true-positive and true-negative results. However, the ""minimize the sum of the error costs"" view seems more intuitive than the ""maximize the sum of the benefits"" view.";


proc sgplot data=GraphData /* sganno=anno2 */;
/* where ThreshP >= 0.01 and ThreshP <= 0.30; */
/*   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); */
   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=PropTruePos / lineattrs=(color=orange); 
/*   series x=ThreshP y=Cost / lineattrs=(color=black); */

/* series x=ThreshP y=PropTrueNeg / lineattrs=(color=orange); */
/*   series x=ThreshP y=Benefit / lineattrs=(color=black pattern=mediumdash); */
   keylegend / location=inside position=top opaque; 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value"; */
   yaxis label = "Proportion of all results";
   run;

ods text="The mirror image of the green line is a meaningful line with a name -- it is the proportion of all the results that are true positive results. As also implied in an earlier graph above, the green line and its mirror image are sigmoid functions that peak (at opposite ends) at &PropTrue on the vertical axis, as shown in the above graph. This is because since only &PctTrue% of the research studies are studying true hypotheses, therefore at most only &PctTrue% of the results can possibly be true positive results or false-negative errors.";
ods text=" ";
ods text="Note that the yellow-orange line on the above graph and the yellow-orange line on the graph that precedes it are the same line. The yellow-orange line on the first of the two graphs is a snippet from the yellow-orange line immediately above. (The two yellow-orange lines appear to have different slopes because the scale on the vertical axis of the second figure is an expanded version of roughly a third of the scale on the vertical axis of the first figure.)";
proc sgplot data=GraphData /* sganno=anno2 */;
/* where ThreshP >= 0.01 and ThreshP <= 0.30; */
   series x=ThreshP y=PropTrueNeg / lineattrs=(color=purple); 
   series x=ThreshP y=PropFalsePosPub / lineattrs=(color=red); 
/*   series x=ThreshP y=PropFalseNeg / lineattrs=(color=green);
   series x=ThreshP y=Cost / lineattrs=(color=black); */

/* series x=ThreshP y=PropTruePos / lineattrs=(color=purple); */
/*   series x=ThreshP y=Benefit / lineattrs=(color=black pattern=mediumdash); */
   keylegend / location=inside position=left opaque; 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
/* xaxis type=linear label = "Threshold p-value"; */
   yaxis label = "Proportion of all results";
   run;

ods text="Above are the red line from the earlier graphs and its mirror image, which is the proportion of all the results that are true negative results. Using the value of PctTrue, which is &PctTrue, the red line peaks at 1 - &PropTrue = &PropFalse if the maximum threshold p-value on the horizontal axis is 1 and if PosPctPub is set at 100. The current value of PosPctPub in this program is &PosPctPub..";
ods text=" ";
ods text="The reason why the lines peak at &PropFalse is that 100% - &PctTrue% = &PctFalse% of the research studies are studying relationships between variables that DON'T exist in the population, so at most &PctFalse% of the results can be true negative results or false-positive results.";
ods text=" ";
ods text="The purple line on the graph three graphs above is a snippet from the purple line on the graph immediately above.";
ods text=" ";
ods text="The line on the following graph is the sum of the ""costs"" and ""benefits"" computed by the program at each threshold p-value. This line will be horizontal at 1.0 on the vertical axis if PosPctPub equals 100. The current value of PosPctPub in this program is &PosPctPub.. This graph is a way of confirming that things are working as expected.";
proc sgplot data=GraphData ;
/* where ThreshP >= 0.01 and ThreshP <= 0.30; */
   series x=ThreshP y=SumCostBen / lineattrs=(color=black); 
   xaxis type=log logbase=10 logstyle=linear 
      label = "Threshold p-value (log scale)"; 
   run;
