/* This program generates the data for the first six graphs in
   appendix M in the paper "The Optimal Threshold p-Value for a
   Scientific Journal" by Donald B. Macnaughton. 

   The output data file from this program is used as input to the
   R program that actually generates the graphs. I use R to 
   generate the graphs because I haven't found a way to make good
   EMF (vector) graphs (for importing into Microsoft Word) out of 
   SAS, but R has a good way to make EMF graphs (with the devEMF
   package). 

   The program also DRAWS the six graphs, but the graphs drawn here
   are intended as simply a check that the program is working 
   properly.

   On some SAS systems, this program may generate an error. If that
   happens, try making the changes to the program given immediately 
   below.

   NOTE: To run this program under SAS OnDemand for Academics,
   use the following steps:
   1. Open the program in the CODE tab.
   2. Note that if the text after the word "type" in step 5 of this
      list of steps consists of simply two quotation marks, then 
      these steps have already been done to this file, so the 
      probematic text has already been removed, so go to step 10.
   3. Click anywhere in the program to ensure the focus is in the
      program.
   4. Press Ctrl+f to open the Find and Replace dialog box.
   5. In the "Find what" field, type "Sasuser." without the quotation
      marks but WITH the period at the end.
   6. Esure that the "Replace with" field is empty.
   7. Click on "Replace All".
   8. Click on "Close".
   9. Press Ctrl+s to save the changed version of the program.
  10. Press F3 to run the program.

   Don Macnaughton December 2023 */

title;
options nodate;
/* Note that the original two-level name in the following DATA 
   statement was "Sasuser" followed by a period followed by 
   "OptGraDa". If you are running this program under SAS OnDemand for
   Academics, you may have removed the "Sasuser" followed by the 
   period using the steps above. The longer name implied that 
   the dataset is saved in a permanent file on the computer so that 
   the R program can read the file to import the data. But trying to 
   save the file this way under SAS OnDemand for Academics causes an 
   error, so that is why it is changed, and no permanent file is 
   saved. 

   In the program listed in the paper, the  dataset is called 
   GraphData, not OptGraDa, but I haven't changed the name here 
   because that would necessitate other changes to the programs that
   read the data file, which might introduce errors. */
data Sasuser.OptGraDa; 

retain PctTrue 30 PosPctPub 90 ntotal 60 meandiff 20 stddev 27;

do ThreshP = 0.01 to 0.3 by 0.001; 
/* Assume 1000 research studies in some field of research. */
   nTrueRels = 1000 * (PctTrue / 100);
   nFalseRels = 1000 - nTrueRels; /* The null hypothesis is true in these cases. */
   nFalsePos = nFalseRels * ThreshP; /* This is by definition of the p-value. */
   nFalsePosPub = nFalsePos * PosPctPub/100;

/* Here are the three obtuse lines of code to compute the power. 
   They are copied from 
https://documentation.sas.com/doc/en/pgmsascdc/9.4_3.2/statug/statug_intropss_sect017.htm
   */
   Ncp = ntotal * 0.5 * 0.5 * meandiff**2 / stddev**2;
   Critval = finv(1-ThreshP, 1, ntotal-2, 0);
   TestPower = sdf('f', Critval, 1, ntotal-2, Ncp); 

   nTruePos = nTrueRels * TestPower;
   nFalseNeg = nTrueRels - nTruePos;

   PropFalsePosPub = nFalsePosPub / 1000;
   PropFalseNeg = nFalseNeg / 1000;

   Cost = PropFalsePosPub + PropFalseNeg;
   output;
   end;

label ThreshP = "Threshold p-value"
      TestPower = "Power of the test"
      PropFalsePosPub = "Proportion of all results that are false-positive errors and are published"
      PropFalseNeg  = "Proportion of all results that are (unpublished) false-negative errors"
      Cost = "Relative total cost of errors assuming equal unit costs of FP and FN";
run;


/* Get necessary values for the graph annotations on figures M.1 and
   M.2 when the threshold p-value is 0.05 and put the values into SAS
   macro variables to be used shortly. 
   The gray lines on these figures are now OMITTED because
   the "sganno=" option is commented out on the PROC SGPLOT
   statements. */
proc sql noprint;
   select   PropFalsePosPub,   PropFalseNeg 
      into :PropFP05 TRIMMED, :PropFN05 TRIMMED 
      from Sasuser.OptGraDa
      where round(ThreshP, 0.001) = 0.05;

/* Get necessary values to draw the arrow on figure M.6 by 
   getting the values associated with the minimum sum of the false-
   positive and fales-negative error costs. */ 
   select   Cost,             ThreshP 
      into :minCost TRIMMED, :OptThresh TRIMMED
      from Sasuser.OptGraDa 
      where Cost = select min(Cost) from Sasuser.OptGraDa;
   quit;

/* Print the values of the macro variables for checking. This will
   include macro variables generated by proc SQL above and any 
   lingering macro values from other programs that were run in this
   SAS session. */
%put _user_;

/* Initialize the annotate macros for drawing on graphs. */
%sganno;

/* Generate annotation data to specify the two grey lines on figure
   M.1. */
data anno1;
   %sgline(drawspace="datavalue", x1=0.05, y1=-0.002, 
      x2=0.05, y2=&PropFP05, linecolor="grey", linethickness=1);
   %sgline(drawspace="datavalue", x1=0.05, y1=&PropFP05, 
      x2=0.01, y2=&PropFP05, linecolor="grey", linethickness=1);
   run;

ods graphics / noborder;

/* Generate figure M.1, the graph with PropFalsePosPub only. 
   The annotation dataset with the specifications for the two lines
   is commented out, so the two gray lines won't appear. */
proc sgplot data=Sasuser.OptGraDa /*sganno=anno1 */;
   series x = ThreshP y=PropFalsePosPub / 
      lineattrs=(pattern=solid color=red);
   series x = ThreshP y=PropFalseNeg   / lineattrs=(pattern=longdash 
      color=white) transparency=1;
   series x = ThreshP y=Cost / lineattrs=(color=white) transparency=1;
   xaxis type=log logbase=10 logstyle=linear 
      label="Threshold p-value (log scale)";
   yaxis label = "False-positive error rate";
   legenditem type=line name="L1" / 
      label="Proportion of all results that are false-positive errors and are published"
      lineattrs=(pattern=solid color=red);
   keylegend "L1" / across = 1 location=inside position=top;
   label PropFalsePosPub = "Proportion of all results that are (unpublished) published false-positive errors"
         PropFalseNeg = "."
         Cost = ".";
   run;

data anno2;
   %sgline(drawspace="datavalue", x1=0.05, y1=-0.002, 
      x2=0.05, y2=&PropFN05, linecolor="grey", linethickness=1);
   %sgline(drawspace="datavalue", x1=0.05, y1=&PropFN05, 
      x2=0.01, y2=&PropFN05, linecolor="grey", linethickness=1);
   run;

/* Generate figure M.2, the graph with PropFalseNeg only. 
   The annotation data set is commented out. */
proc sgplot data=Sasuser.OptGraDa /* sganno=anno2 */;
   series x = ThreshP y=PropFalsePosPub / 
      lineattrs=(pattern=shortdash color=white) transparency=1;
   series x = ThreshP y=PropFalseNeg / 
      lineattrs=(pattern=solid color=green) transparency=0;
   series x = ThreshP y=Cost / lineattrs=(color=white) transparency=1;
   xaxis type=log logbase=10 logstyle=linear 
      label="Threshold p-value (log scale)";
   yaxis label = "False-negative error rate";
   legenditem type=line name="L1" / 
      label="Proportion of all results that are (unpublished) false-negative errors"
      lineattrs=(pattern=solid color=green);
   keylegend "L1" / across = 1 location=inside position=top;
   label PropFalsePosPub = "."
         PropFalseNeg = "Proportion of all results that are false-negative errors"
         Cost = ".";
   run;

/* Generate figure M.3 with both error rates. */
proc sgplot data=Sasuser.OptGraDa;
   series x = ThreshP y=PropFalsePosPub   / 
      lineattrs=(pattern=solid color=red);
   series x = ThreshP y=PropFalseNeg   / 
      lineattrs=(pattern=solid color=green);
   series x = ThreshP y=Cost / lineattrs=(color=white) transparency=1;
   xaxis type=log logbase=10 logstyle=linear label="Threshold p-value (log scale)";
   yaxis label = "Error rate";
   legenditem type=line name="L1" / 
      label="Proportion of all results that are false-positive errors and are published"
      lineattrs=(pattern=solid color=red);
   legenditem type=line name="L2" / 
      label="Proportion of all results that are (unpublished) false-negative errors"
      lineattrs=(pattern=solid color=green);
   keylegend "L1" "L2" / across = 1 location=inside position=top;
   label PropFalsePosPub = "Proportion of all results that are false-positive errors and are published"
         PropFalseNeg = "Proportion of all results that are (unpublished) false-negative errors"
         Cost = ".";
   run;

/* Generate figure M.4 with both costs but without the sum of the costs. */
proc sgplot data=Sasuser.OptGraDa;
   series x = ThreshP y=PropFalsePosPub   / 
      lineattrs=(pattern=solid color=red);
   series x = ThreshP y=PropFalseNeg   / 
      lineattrs=(pattern=solid color=green);
   series x = ThreshP y=Cost / lineattrs=(color=white) transparency=1;
   xaxis type=log logbase=10 logstyle=linear 
      label="Threshold p-value (log scale)";
   yaxis label = "Cost";
   legenditem type=line name="L1" / label="False-positive error costs" 
      lineattrs=(pattern=solid color=red);
   legenditem type=line name="L2" / label="False-negative error costs" 
      lineattrs=(pattern=solid color=green);
   keylegend "L1" "L2" / across = 1 location=inside position=top;
   label PropFalsePosPub = "Proportion of all results that are false-positive errors and are published"
         PropFalseNeg = "Proportion of all results that are (unpublished) false-negative errors"
         Cost = ".";
   run;

/* Generate figure M.5 with the sum of the costs. */
proc sgplot data=Sasuser.OptGraDa;
   refline 0;
   series x = ThreshP y=PropFalsePosPub   / 
      lineattrs=(pattern=solid color=red);
   series x = ThreshP y=PropFalseNeg   / 
      lineattrs=(pattern=solid color=green);
   series x = ThreshP y=Cost / lineattrs=(color=black);
   xaxis type = log logbase=10 logstyle=linear 
      label="Threshold p-value (log scale)";
   yaxis label = "Cost";
   legenditem type=line name="L1" / label="False-positive error costs" 
      lineattrs=(pattern=solid color=red);
   legenditem type=line name="L2" / label="False-negative error costs" 
      lineattrs=(pattern=solid color=green);
   legenditem type=line name="L3" / label="Sum of costs" 
      lineattrs=(pattern=solid color=black);
   keylegend "L1" "L2" "L3" / across = 1 location=inside position=top;
   run;

/* Generate the annotation data for the arrow on figure M.6. */
data anno3;
   %sgarrow(drawspace="datavalue", x1=&OptThresh, y1=&minCost, 
      x2=&OptThresh, y2=0, linecolor="black", linethickness=1);
   run;

/* Generate figure M.6, which is figure M.5 with the arrow. */
proc sgplot data=Sasuser.OptGraDa sganno=anno3;
   series x = ThreshP y=PropFalsePosPub   / 
      lineattrs=(pattern=solid color=red);
   series x = ThreshP y=PropFalseNeg   / 
      lineattrs=(pattern=solid color=green);
   series x = ThreshP y=Cost / lineattrs=(color=black);
   xaxis type = log logbase=10 logstyle=linear 
      label="Threshold p-value (log scale)";
   yaxis label = "Cost";
   legenditem type=line name="L1" / label="False-positive error costs" 
      lineattrs=(pattern=solid color=red);
   legenditem type=line name="L2" / label="False-negative error costs" 
      lineattrs=(pattern=solid color=green);
   legenditem type=line name="L3" / label="Sum of costs" 
      lineattrs=(pattern=solid color=black);
   keylegend "L1" "L2" "L3" / across = 1 location=inside position=top;
   run;

/* Put the values of the macro variables into a line in a new dataset
   for transfer to R, which needs the values to add the annotations
   to the figures. */

data Sasuser.MacVals;
   PropFP05 = &PropFP05;
   PropFN05 = &PropFN05;
   MinCost = &minCost;
   OptThresh = &OptThresh;
   output;
   run;
